#!/usr/local/bin/perl -w

#### LAST UPDATED: Nov15, 2002 ####
# filter1.pl version 0.1

###########################################################################
#
# This program removes the data within angular braces in the context of the
# SOURCE file. This data is non-contextual and is thus eliminated before the
# tagging phase along with the angular braces. The output is placed
# in the DESTINATION file. The data which is filtered out is placed in the
# FILTER file. 
#
# USAGE :  filter1.pl [OPTIONS] FILTER DESTINATION SOURCE
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }
# if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}


# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 2 ) {
    &usage();
    exit;}

$filterfl = shift();
$opfl = shift();
$inpfl = shift();

if(-f $opfl) {  
    print "Destination file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
	print "Overwriting the file.\n"; }
    else{
	exit; } }
 
if(-f $filterfl){ 
    print "Filter file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
        print "Overwriting the file.\n"; }
    else{
        exit; } } 

# Open the various files
open(INP,"<$inpfl");
open(DB,">$filterfl");
open(OP,">$opfl");

$incontext = 0;
# Read lines from input file
while($ln = <INP>) {
    # Set variable based on whether line part of context or not
    if($ln =~ /^\s*<context>\s*$/) {
	$incontext = 1;
	print OP $ln;
	next; }
    if($ln =~ /^\s*<\/context>\s*$/) {
	$incontext = 0; 
	print OP $ln;
	next; }
    
    # Process only if line part of context
    if($incontext){
	# Ignore blank lines
	if($ln =~ /^\s*$/){
	    next;}

	# Eliminate xml tags (demarcated by angular braces)
	while($ln =~ /(<[^>]+>)/){
	    $xtag = $1;
	    # Print data eliminated in filter file
	    print DB "$xtag\n"; 
	    # Change Mar 8
   	    # if ($xtag eq "<\/s>") {
		# $ln =~ s/<\/s>/\n/; 
		# next; }	
	    # End change
	    # CHANGE
	    # $ln =~ s/$xtag/ /;} }
	    $ln =~ s/<[^>]+>/ /;} }

    print OP $ln; }

# Subroutine to print help message
sub help{
    print "Usage: filter1.pl [OPTIONS] FILTER DESTINATION SOURCE\n\n"; 
    print "This program removes the data within angular braces in the context of the SOURCE file. This data is non-contextual and is thus eliminated before the tagging phase along with the angular braces. The output is placed in the DESTINATION file. The data which is filtered out is placed in the FILTER file.\n";
    print "OPTIONS:\n\n";
    print "  --help             Prints the help message.\n";
    print "  --version          Prints the version number.\n\n"; }

sub usage{
    print "Usage: filter1.pl [OPTIONS] FILTER DESTINATION SOURCE\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type filter1.pl --help for help.\n";}
# function to output the version number
sub showVersion
{
    print STDERR "filter1.pl      -        version 0.1\n";
    print STDERR "A component of posSenseval version 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad Ted Pedersen\n";
}

