#!/usr/local/bin/perl -w

#### LAST UPDATED: Oct19, 2002 ####
# radical.pl version 0.1

###########################################################################
#
# This program prints all those head words whose pre-assigned tag has been
# updated by the Brill Tagger. It takes as input a file (SOURCE) which has
# the line as after pre-tagging and the line as after the Brill tagger has
# tagged it. 
# This program prints data corresponding to radical changes only i,e, cases
# where the updated tag is in a different class altogether as compared
# to the preassigned tag. For eg. an updation frm noun to verb etc...
# In this package, the output file of tstchngtag.pl is used as input (SOURCE)
# for this program.
# The number of lines in the output file is the number of mistaggings.
# The ouput goes to stdout unless redirected.
#
# USAGE : radical.pl [OPTIONS] SOURCE
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help )
{
    &help();
    exit;
}
# if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

# Check if commandline options have been provided
# If not print usage information
if ( $#ARGV < 0 )
{
    &usage();
    exit;
}

# Read lines from the source file (Only the pretagged lines are read here)
while(<>)
{
    # If line is blank read next line
    if(/^\s*$/)
    { next; }
    # Capture the pre-tag 
    if(/\/\/([A-Z])/)
    {
	$trg = $1;
    }
    # If input is erroneous, exit
    if(!($trg))
    {
        print "trg exit\n";
        exit;
    }

    # Read the Brill tagged line 
    while(<>)
    {
	# If line is blank read next line
        if(/^\s*$/)
        { next; }
	else
        { last; }
    }
    # Capture the headword and updated tag 
    if(/([^\s\/]+)\/\/([A-Z])/)
    {
	$wd = $1;
	$src = $2;
    } 
    # If input is erroneous, exit
    if(!($src))
    {
        print "src exit\n";
        exit;
    }

    # Check if pre-tag is in the same class as the Brill tag
    # If yes goto next line, else print relevant data 
    if($src eq $trg)
    { next; }
    else
    {
	print "$wd \t $src \t $trg\n";
    }
    
}

# Subroutine to print help message
sub help
{
    print "Usage: radical.pl [OPTIONS] SOURCE \n\n";
    print "This program prints all those head words whose pre-assigned tag has been updated by the Brill Tagger. It takes as input a file (SOURCE) which has the line as after pre-tagging and the line as after the Brill tagger has tagged it. The program prints data corresponding to radical changes only i,e, cases where the updated tag is in a different class altogether as compared to the preassigned tag. For eg. an updation from noun to verb etc. The first column is the pretagged word, the second column is the tag assigned by the Brill tagger and the third column is the pre-tag.The ouput goes to stdout unless redirected.\n\n";

    print "OPTIONS:\n\n";
    print "  --help             Prints the help message.\n";
    print "  --version          Prints the version number.\n\n";
}

sub usage
{
    print "Usage: radical.pl [OPTIONS] SOURCE \n";
    AskHelp();
}
# Message describing way to get help
sub AskHelp
{
    print "Type radical.pl --help for help.\n";
}	
# function to output the version number
sub showVersion
{
    print STDERR "radical.pl      -        version 0.1\n";
    print STDERR "A component of posSenseval version 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad Ted Pedersen\n";
}

