#!/usr/local/bin/perl -w

#### LAST UPDATED: Feb, 2003 ####
# xml-back.pl version 0.1

###########################################################################
#
# This program adds the xml tags from the SOURCE1 to the lines in SOURCE2 
# and places the resultant lines in DESTINATION file. The number of non-xml
# tokens in the SOURCE1 are compared with the number of tokens in the SOURCE2 
# file. Any mismatch causes the relevant lines of the two files to be 
# printed in the FILTER file. This file is empty if there is perfect 
# synchronisation among the two files.
# Other functions include:
#
# a. Non-contextual data within square brackets is put in angular braces
# b. Non-contextual data within curly braces (found in Senseval-1 data)
#    also placed in angular braces. An exception to thi sbeing typo
#    error information in curly braces; the information is replaced
#    by the correct spelling.
# c. Character references which were eliminated during pre-processing are
#	put in angular braces 
#
# USAGE : xml-back.pl [OPTIONS] FILTER DESTINATION SOURCE1 SOURCE2 
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }
# if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

# check if command line options have been provided
# If not print usage information
if ( $#ARGV < 3 ) {
    &usage();
    exit; }

# Store the files specified by the command line 
$dbxmlfl = shift @ARGV;
$opfl = shift @ARGV;
$xmlfl = shift @ARGV;
$tgdfl = shift @ARGV;

# Check if DESTINATION and FILTER files specified already exist
if(-f $opfl) {
    print "Destination file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
        print "Overwriting the file.\n"; }
    else {
        exit; } }

if(-f $dbxmlfl) {
    print "Filter file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
        print "Overwriting the file.\n"; }
    else {
        exit; } }

# Open files involved
open(XML, "<$xmlfl") or die "could not open file=== xmlfl";
open(TGD, "<$tgdfl") or die "could not open file--- tgdfl";
open(OUTPUT, ">$opfl") or die "could not open file";
open(DBX, ">$dbxmlfl") or die "could not open file";

# Read lines from the SOURCE file - line by line
$lnno = 0;
$errfl = 0;
while(<XML>) {
    # Increment line number of the line being processed in SOURCE1
    # displayed on stdout and FILTER in case of mismatch
    $lnno++;
    # No new line character should exist within angular braces
    if(/^\s?<!DOCTYPE corpus SYSTEM\s?\n/) {
	s/^(\s?<!DOCTYPE corpus SYSTEM\s?)\n/$1/;
	$exln = <XML>;
	$_ = $_.$exln; }

    # Jul16
    # Senseval-1 data has few instances with a lone square bracket
    # float-v.800152 has token `Horrors]' which is garbled and hence removed
    s/^Horrors\] / /;
    # promise-n.800121 and shirt-n.800409 have a lone ] which is removed
    s/(The people have won, Aylwin is President)\]/$1/;
    s/(^\`No, sir)\]/$1/;
  
    # Jul 16 
    # contextual data within square braces is placed between curved braces
    # s/\[(\s?[\w\.]+\s[\w\.]+[^\]]*)\]/\($1\)/g;
    s/\[(\s*[^\]\s]+\s+[^\]\s]+\s*[^\]]*)\]/\($1\)/g;
    # s/\[\s*(([^\]\s]+\s+)*)\]/ ( $1 ) /g; 

    # typo information as in {} braces is replaced by correct spelling
    s/\{typo bad=\"[^\"]*\",\s*good=\"([^\"]*)\"[^\}]*\}/ $1 /g;
    # Non-contextual data within square brackets is put in angular braces
    s/(\[[^\]]+\])/<$1>/g;
    # Eliminating data in {}
    # s/(\{[^\}]*\})/<$1>/g;
    while(/(\{[^\}]*\})/g)
    {
	$curly = $1;
	$oldcurly = $curly;
	$curly =~ s/\s+/curlyspace/g;
	$curly =~ s/\{/startcurly/;
	$curly =~ s/\}/endcurly/;
	s/$oldcurly/$curly/g;
    }
    s/startcurly\s*/<\{/g;
    s/\s*endcurly/\}>/g;

    # Jul16
    s/(^|[^<])\)/$1 \) /g;
    s/(^|[^<])\(/$1 \( /g;
   
    # CHANGE Jul6
    # -- is tokenised to --
    s/--/ -- /g;
    # Tokenize non compound forming `-'
    # CHANGE Jul15
    # s/(\w)(-+)(\s|$)/$1 $2$3/g;
    # s/(^|\s)(-+)(\w)/$1$2 $3/g;
    # Jul 16 comments
    # s/([a-zA-Z0-9])(-+)(\s|$)/$1 $2$3/g;
    # s/(^|\s)(-+)([a-zA-Z0-9])/$1$2 $3/g;
    # Jul16
    # s/(\'[^\s]+)-([^\s]+)/$1 - $2/g;

    # s/([^-])-([^-])/$1 - $2/g;
    # s/([a-zA-Z])\s+-\s+([a-zA-Z])/$1-$2/g;
    
    # Replace those character references which were not eliminated during pre-processing
    # by appropriate characters
    while(/&mdash;/g) {
	s/&mdash;/ -- /g; }
    while(/&ndash;/g) {
	s/&ndash;/ - /g; }
    while(/&([be]|[lr]d)?quo;/g) {
	s/&([be]|[lr]d)?quo;/ " /g; }
    while(/&amp;/g) {
        s/&amp;/ & /g; }
    while(/&lt;/g){
        s/&lt;/ < /g;}
    while(/&gt;/g){
        s/&gt;/ > /g;}
    while(/&apos;/g){
        s/&apos;/ ' /g;}
    while(/&lsquo;/g){
        s/&lsquo;/ ' /g;}
    while(/&rsquo;/g){
        s/&rsquo;/ ' /g;}
    # Jul16
    while(/(^|[^<])&lsqb;([^>]|$)/g){
        s/&lsqb;/ [ /g;}
    while(/[^<]&rsqb;[^>]/g){
        s/&rsqb;/ ] /g;}
    while(/&num;/g){
        s/&num;/ \# /g;}
    while(/&lcub;/g){
        s/&lcub;/ \{ /g;}
    while(/&rcub;/g){
        s/&rcub;/ \} /g;}
    # Jul16
    while(/&equals\s*;/g){
	s/&equals\s*;/ = /g;}

    # Put the remaining character references in angular braces
    # CHANGE May
    # s/(^|[^<])(&[^&;\.\s]*[;\.])/$1 <$2> /g;
    # CHANGE May30
    s/(&[^&;\.\s]*[;\.])/ <$1> /g;
    # s/(^|[^<])\s*(&[^&;\.\s]*[;\.])/$1 <$2> /g;
    
    # If the SOURCE line is one complete XML tag, print it to DESTINATION file
    if(/^\s*<[^>]+>\s*$/) {
	print OUTPUT;
	print OUTPUT "\n";
	next; }
    # Ignore empty lines
    if(/^\s*$/) {
	next; }
   
    # tokenise punctuations
    s/</ </g;
    s/>/> /g;

    # Keep the curly tokens together
    # s/\s*<&curlyspace\;>\s*/<&curlyspace\;/g;
    #### CHANGE May30
    s/(^|\s)([^\s&<\;]+)\;/$1$2 \; /g;
    #### s/(^|[^<;]+);/$1 ; /g;

    s/(^|[^<]):/$1 : /g;
    # CHANGE
    s/(^|[^<]),/$1 , /g;
    s/(^|[^<])\?/$1 \? /g;
    # Jul16
    # s/(^|[^<])\./$1 \. /g;
    s/\./ \. /g; 

    s/\"/ \" /g;
    # CHANGE FEB17
    while(s/(\{[^\s\}]*)\s+\"\s*/$1\"/g) {;}
    # CHANGE
    # s/(^|[^<])\'/$1 \'/g;
    # s/(^|[^<])\`/$1 \` /g;
    # Jul16
    s/(^|[^<\'])\'/$1 \'/g;
    s/(^|[^<])(\`(\`)?)/$1 $2 /g;

    # Jul16
    # s/(^|[^<])\)/$1 \) /g;
    # s/(^|[^<])\(/$1 \( /g;

    s/([^<])\[/$1 \[/g;
    s/\]([^>])/\] $1/g;
    # CHANGE MAY30
    s/(^|[^<])&/$1 & /g;
    # CHANGE MAY30
    # s/& (\w+) ;/&$1;/g;
    s/&\s*(\w+)\s*\;/&$1\;/g;

    s/\// \/ /g;
    # s/([^-])-([^-])/$1 - $2/g;
    # s/([a-zA-Z])\s+-\s+([a-zA-Z])/$1-$2/g;
    s/!/ ! /g;
    s/\+/ \+ /g;

    s/(^|[^<])\$/$1 \$ /g;
    s/(^|[^<])%/$1 % /g;
    s/(^|[^<])\#/$1 \# /g;

    s/\226/ \226 /g;
    s/\227/ \227 /g;
    s/[^<][^<]/  /g;
    s/\*/ \* /g;
    s/\{/ \{ /g;
    s/\}/ \} /g;
    # CHANGE FEB17
    s/<\s*{\s*/<\{/g;
    s/\s*\}\s*>/\}>/g;
    # END CHANGE
    s/\\ \// \\\/ /g;
    s/ \/  \/ /\//;
    if (/\/ [^\s]+[\]>]/) {
	s/ \/ ([^\s]+[\]>])/\/$1/g; }

    # Jul16
    s/([a-zA-Z0-9])(-+)(\s|$)/$1 $2$3/g;
    s/(^|\s)(-+)([a-zA-Z0-9])/$1$2 $3/g;
    # Jul16
    s/(\'[^\s]+)-([^\s]+)/$1 - $2/g;

    # DBUG
    if (/Equation for success/) {
	   #print;
    }
    
    # Initialise word counter of XML line	
    $cntr1=0;
    # CAPTURE TOKENS SEPARATED BY WHITE SPACE IN VARIABLE
    for $chunk1 (split) {
	$cntr1++;
	$xmlwd[$cntr1]=$chunk1; }
    
    # Initialise XML tag counter and empty line flag	
    $xmlcnt=0;
    $lnempty=1;
    # Loop till all tokens in XML line exhausted
    for($i=1;$i<($cntr1+1);$i++) {
	# If token is xml tag, print
	if($xmlwd[$i] =~ /<[^\s]+>|\226|\227|\\\//) {
	    # CHANGE FEB17
	    $xmlwd[$i] =~ s/curlyspace/ /g;
	    print OUTPUT "$xmlwd[$i] ";
	    # Increment xml tag ctr
	    $xmlcnt++;
	    next; }
	# Print xml tags with multiple tokens within it	
	if($xmlwd[$i] =~ /<[^>]+$/) {
	    print OUTPUT "$xmlwd[$i] ";
	    $xmlcnt++;
	    $i++;
	    while($xmlwd[$i] =~ /[^>]$/) {
		print OUTPUT "$xmlwd[$i]";
		$xmlcnt++;
		$i++ }
	    print OUTPUT "$xmlwd[$i] ";
	    $xmlcnt++;
	    next; }
	
	# Read new line from TAGGED file only if XML line is devoid of any non-xml tokens
	if($lnempty==1) {
	    # Read new line from TAGGED file
	    $tgdln = <TGD>;
	    $tgdln =~ s/([^\s])\/\//$1\//;
	    # If tagged line is empty read another line	
	    while($tgdln =~ /^\s*$/) {
		$tgdln = <TGD>;
		$tgdln =~ s/([^\s])\/\//$1\//; }
	    $cntr2=0;
	    @tgdwd="";
	    # CAPTURE TOKENS SEPARATED BY WHITE SPACE IN VARIABLE
	    for $chunk1 (split(/\s/,"$tgdln")) {
		$cntr2++;
		$tgdwd[$cntr2]=$chunk1; }
	    $tgdwdno = @tgdwd; }
	
	# In case of mismatch of non-xml tokens print error message with line number in XML file
	if($i-$xmlcnt+1>$tgdwdno) {
	    $errfl = 1;
	    print "Error at $lnno : accessing extra element from tagged line\n";
	    print DBX "TGD: $tgdln \n $_\n\n"; }
	else {
	    $lnempty=0;
	    # Print non-xml token to DESTINATION
	    # print OUTPUT "$tgdwd[$i-$xmlcnt] ";
	    # Convert text to lower case
	    # $lcwd = lc($tgdwd[$i-$xmlcnt]);
	    if( $tgdwd[$i-$xmlcnt] =~ /^\/\//) {
		$tgdwd[$i-$xmlcnt] =~ s/^\//<&frasl>/; }
	    @arrcase = split(/\//, $tgdwd[$i-$xmlcnt]);
	    $arrcase[0] = lc($arrcase[0]);
	    $lcwd = "$arrcase[0]"."\/"."$arrcase[1]";
	    
	    print OUTPUT "$lcwd "; } } 
    print OUTPUT "\n"; }

if($errfl == 0)		{ unlink "$dbxmlfl"; }

# Subroutine to print help message
sub help {
    print "Usage: xml-back.pl [OPTIONS] FILTER DESTINATION SOURCE1 SOURCE2\n\n"; 
    print "This program adds the xml tags from the SOURCE1 file to the SOURCE2 file and places the resultant lines in DESTINATION file. Any mismatch causes the relevant lines of the two files to be printed in the FILTER file. Other functions include...\n\n";
    print "\ta. Non-contextual data within square brackets is put in angular braces\n";
    print "\tb. Character references which were eliminated during pre-processing are put in angular braces\n\n";
    print "OPTIONS:\n\n";
    print "  --help             Prints the help message.\n"; 
    print "  --version          Prints the version number.\n\n"; }

sub usage {
    print "Usage: xml-back.pl [OPTIONS] FILTER DESTINATION SOURCE1 SOURCE2\n"; 
    AskHelp(); }
# Message describing way to get help
sub AskHelp {
    print "Type xml-back.pl --help for help.\n"; }
# function to output the version number
sub showVersion
{
    print STDERR "xml-back.pl      -        version 0.1\n";
    print STDERR "A component of posSenseval version 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad Ted Pedersen\n";
}

