#!/usr/local/bin/perl -w

######################### LAST UPDATED: Jan,2003 ##########################

# xmlise.pl Version 0.1

###########################################################################
#
# This program formats the part of speech tags into angular braces. The 
# formatted form of the SOURCE file is placed in the DESTINATION file.
# The `<' and `>' symbols are first converted back to the sgml format and
# placed in angular braces; thus transformed to `<&lt;>' and `<&gt;>', 
# respectively.
#
# USAGE :  xmlise.pl [OPTIONS] DESTINATION SOURCE
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help )
{
    &help();
    exit;
}
# if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

# check if command line options have been provided
# If not print usage information
if ( $#ARGV < 1 )
{
    &usage();
    exit;
}

# Capture file names for command line and open files
$opfl = shift(@ARGV);
$inpfl = shift(@ARGV);

if(-e $opfl)
{
    print "Destination file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") 
    {
        print "Overwriting the file.\n";
    }
    else
    {
        exit;
    }
}

open(I,"$inpfl") or die "Could not open file inp\n";
open(O,">$opfl") or die "Could not open file op\n";

# Read data from SOURCE - line by line
while (<I>)
{
    # Substituting < and > with &lt; and &gt;
    s/(^|\s)<(\/[^>\s]+\s)/$1<&lt\;>$2/g;
    s/(^|\s)>(\/[^>\s]+\s)/$1<&gt\;>$2/g;
    # Format POSZ tags into angular braces
    # s/([^\s<\[]+)\/([\w\$]+)/$1 <p="$2"\/>/g ;
    s/([^\s<\[]+)\/([^\s<>]+)/$1 <p="$2"\/>/g ;
    # Remove head space
    s/(<head[^>]*>)\s*/$1/g;
    s/ <\/head>/<\/head>/g;
    s/(<sat[^>]*>)\s*/$1/g;
    s/ <\/sat>/<\/sat>/g;

    # Print to DESTINATION file
    print O;
}

# Subroutine to print help message
sub help
{
    print "Usage: xmlise.pl [OPTIONS] SOURCE DESTINATION \n\n";
    print "This program formats the part of speech tags into angular braces. The formatted form of the SOURCE file is placed in the DESTINATION file.\n\n";
    print "OPTIONS:\n\n";
    print "  --help             Prints the help message.\n";
    print "  --version          Prints the version number.\n\n";
}

sub usage
{
    print "Usage: xmlise.pl [OPTIONS] SOURCE DESTINATION \n";
    AskHelp();
}
# Message describing way to get help
sub AskHelp
{
    print "Type xmlise.pl --help for help.\n";
}
# function to output the version number
sub showVersion
{
    print STDERR "xmlise.pl      -        version 0.1\n";
    print STDERR "A component of posSenseval version 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad Ted Pedersen\n";
}

